<?php
/**
 * @file
 * Survey Builder interface code
 */

/**
 * Create/edit survey page title callback
 */
function survey_builder_create_title($form_id) {
  if ($form_id != SURVEY_BUILDER_NEW) {
    return t('Edit Survey');
  }
  return t('Add Survey');
}

/**
 * Create/edit survey page callback
 */
function survey_builder_create($form_id) {

  drupal_add_css(drupal_get_path('module', 'survey_builder') . '/theme/css/survey.css', array('group' => CSS_DEFAULT));

  $options = array('type' => 'file', 'scope' => 'footer');
  drupal_add_js(drupal_get_path('module', 'survey_builder') . '/theme/js/jquery.tools.min.js', $options);
  drupal_add_js(drupal_get_path('module', 'survey_builder') . '/theme/js/survey.js', $options);

  module_load_include('inc', 'form_builder', 'includes/form_builder.admin');
  module_load_include('inc', 'form_builder', 'includes/form_builder.cache');

  return array(
    '#type' => 'container',
    '#attributes' => array(),
    'clone' => array(
      '#markup' => '<a href="#" class="button survey-builder-clone-button">Clone</a>',
    ),
    'survey' => form_builder_interface('survey', $form_id),
  );

}

/**
 * Prepare a field from a form builder form array for loading
 */
function survey_builder_load_process_field(&$field, $survey_response_id = FALSE) {

  module_load_include('inc', 'survey_builder', 'includes/survey_builder.question');

  if (isset($field['#type']) && $field['#type'] == 'fieldset') {
    // Recursively process any fields within the fieldset
    foreach ($field as $key => &$value) {
      if (strpos($key, '#') !== 0) {
        survey_builder_load_process_field($value, $survey_response_id);
      }
    }
  }
  else {
    $question = question_load($field['#question_id']);
    if (!$question) {
      watchdog('survey_builder', 'Unable to load question: %id',
        array('%id' => $field['#question_id']), WATCHDOG_ERROR);
      return;
    }

    // Set question label in field array
    $question->field['#title'] =  $question->label;

    unset($field['#options']);

    // Merge question values into field
    $field = array_merge($question->field, $field);

    // If a response ID has been supplied then attempt to load this question's response
    if ($survey_response_id !== FALSE) {

      $response = question_response_load($survey_response_id, $field['#key']);
      if (!$response) {
        return;
      }
      elseif (is_array($response)) {
        $field['#default_value'] = array();
        foreach ($response as $value) {
          $field['#default_value'][] = $value->value;
        }
      }
      elseif (!is_null($response->value_key)) {
        $field['#default_value'] = $response->value_key;
      }
      else {
        $field['#default_value'] = $response->value;
      }

      if ($field['#type'] == 'checkboxes' && !is_array($field['#default_value'])) {
        $field['#default_value'] = array($field['#default_value']);
      }

    }

  }

}

/**
 * Survey save callback
 */
function survey_builder_save($form_id) {

  module_load_include('inc', 'form_builder', 'includes/form_builder.api');
  module_load_include('inc', 'form_builder', 'includes/form_builder.cache');

  $form = form_builder_cache_load('survey', $form_id);
  form_builder_save_form($form, 'survey', $form_id);

  drupal_goto('surveys');

}

/**
 * Prepare a field from a form builder form array for saving
 */
function survey_builder_save_process_field(&$field) {

  module_load_include('inc', 'survey_builder', 'includes/survey_builder.question');

  if (!is_array($field)) {
    return;
  }
  else if ($field['#type'] == 'fieldset') {
    // Recursively process any fields within the fieldset
    foreach ($field as $key => &$value) {
      if (strpos($key, '#') !== 0) {
        survey_builder_save_process_field($value);
      }
    }
  }
  else {
    $master = FALSE;
    $question_field = $field;

    // These values will be set
    $elements = array('#question_id', '#title', '#key', '#form_builder', '#weight', '#required', '#master');

    if (!isset($field['#question_id'])) {

      // New question. Save it and store the new question's ID
      $id = question_save($field['#title'], $field);
      if ($id === FALSE) {
        watchdog('survey_builder', 'Unable to save new question: %field',
          array('%field' => print_r($field, TRUE)), WATCHDOG_ERROR);
        continue;
      }

      $master = TRUE;

    }
    else {

      $id = $field['#question_id'];

      $question = question_load($id);
      if ($question === FALSE) {
        watchdog('survey_builder', 'Unable to load existing question: %field',
          array('%field' => print_r($field, TRUE)), WATCHDOG_ERROR);
        continue;
      }

      if (isset($field['#master']) && $field['#master']) {
        $master = TRUE;
        unset($field['#master']);

        $question->label = $field['#title'];
        $question->field = $field;
        $question->save();
      }
      else {
        $question_field = $question->field;
      }

    }

    $field = array(
      '#question_id' => $id,
      '#title' => $field['#title'],
      '#key' => $field['#key'],
      '#form_builder' => array(
        'element_id' => $field['#form_builder']['element_id'],
        'element_type' => 'question',
        'configurable' => TRUE,
        'removable' => TRUE,
        'parent_id' => 0,
      ),
      '#weight' => (isset($field['#weight'])) ? $field['#weight'] : 0,
      '#required' => (isset($field['#required'])) ? $field['#required'] : 0,
      '#master' => $master,
    );

    // Duplicate all other values not set
    if (is_array($field)) {
      foreach ($question_field as $key => $value) {
        if (!in_array($key, $elements)) {
          $field[$key] = $value;
        }
      }
    }

  }

}

/**
 * Complete survey form callback
 */
function survey_builder_complete_form($form, &$form_state, $form_id) {

  module_load_include('inc', 'survey_builder', 'includes/survey_builder.survey');

  $survey = survey_load($form_id);
  if (!$survey) {
    drupal_not_found();
    exit();
  }

  drupal_set_title($survey->title);

  $form_state['survey_id'] = $form_id;

  $form = $survey->fields;
  foreach ($form as $key => &$field) {
    survey_builder_load_process_field($field);
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
    '#weight' => 1000,
  );

  return $form;

}

/**
 * Complete survey form submit handler
 */
function survey_builder_complete_form_submit($form, &$form_state) {

  // Save survey response entity and associated question response entities
  module_load_include('inc', 'survey_builder', 'includes/survey_builder.survey');

  $response_id = survey_response_save($form_state['survey_id'], $form_state['input'], $form_state);
  if (!$response_id) {
    drupal_set_message(t('Unable to save survey response'), 'error');
  }

  $form_state['redirect'] = 'surveys';

}

/**
 * View completed survey page callback
 */
function survey_builder_response_view($response_id) {

  module_load_include('inc', 'survey_builder', 'includes/survey_builder.survey');

  $response = survey_response_load($response_id);
  if (!$response) {
    return MENU_NOT_FOUND;
  }

  $survey = survey_load($response->survey_id);
  if (!$survey) {
    return MENU_NOT_FOUND;
  }

  $header = array(
    t('Question'),
    t('Response')
  );

  $rows = survey_builder_response_process_fields($survey->fields, $response->pid);

  return theme('table', array('header' => $header, 'rows' => $rows));

}

function survey_builder_response_process_fields($fields, $survey_response_id) {

  $rows = array();

  // Sort the fields based on weight
  uasort($fields, 'element_sort');

  foreach ($fields as $key => &$field) {

    if (strpos($key, '#') === 0) {
      continue;
    }
    elseif (isset($field['#type']) && $field['#type'] == 'fieldset') {
      // Recursively process any fields within the fieldset
      $rows = array_merge($rows, survey_builder_response_process_fields($field, $survey_response_id));
    }
    else {

      survey_builder_load_process_field($field, $survey_response_id);

      $value = '';

      if (isset($field['#default_value'])) {
        if (is_array($field['#default_value'])) {
          $values = array();
          foreach($field['#default_value'] as $item) {
            if (!empty($field['#options'][$item])) {
              $values[] = $field['#options'][$item];
            }
          }
          $len = count($values);
          if ($len > 1) {
            $value = implode(', ', $values);
          }
          elseif ($len === 1) {
            $value = $values[0];
          }
          else {
            $value = '';
          }
        }
        elseif (isset($field['#options'])) {
            $value = $field['#options'][$field['#default_value']];
        }
        else {
            $value = $field['#default_value'];
        }
      }

      $rows[] = array(check_plain($field['#title']), check_plain($value));

    }

  }

  return $rows;

}

/**
 * Survey delete page callback
 */
function survey_builder_delete($form, &$form_state, $form_id) {

  module_load_include('inc', 'survey_builder', 'includes/survey_builder.survey');

  $survey = survey_load($form_id);
  if (!$survey) {
    return MENU_NOT_FOUND;
  }

  $form['pid'] = array('#type' => 'value', '#value' => $survey->pid);

  return confirm_form($form,
    t('Are you sure you want to delete %title?', array('%title' => $survey->title)),
    'surveys',
    t('This action cannot be undone.'),
    t('Delete'),
    t('Cancel')
  );

}


/**
 * Delete survey
 */
function survey_builder_delete_submit($form, &$form_state) {

  if ($form_state['values']['confirm']) {

    $survey = survey_load($form_state['values']['pid']);

    if (survey_delete($survey->pid) === FALSE) {
      watchdog('survey_builder', 'Failed to delete %title.', array('%title' => $survey->title));
      drupal_set_message(t('Unable to delete %title', array('%title' => $survey->title)), 'error');
    }
    else {
      watchdog('survey_builder', 'Deleted %title.', array('%title' => $survey->title));
      drupal_set_message(t('%title has been deleted.', array('%title' => $survey->title)));
    }

  }

  $form_state['redirect'] = 'surveys';

}
